﻿using UnityEngine;
using System.Collections;

[ System.Serializable ]
public class BodyParts
{
	public SimpleBound head;
	public SimpleBound body;
	public SimpleBound frontLeg;
	public SimpleBound backLeg;
}

public class PlayerController : MonoBehaviour 
{
	public delegate void OnPlayerScore ();
	public event OnPlayerScore 								onPlayerScore;
	
	public string 											name;
	public BodyParts										bodyParts;
	
	public float 											speed;
	public float											speedOnAir;
	public float											speedOnAirConstant;
	public float 											jumpHeight = 50f;
	public float 											jumpTime = 0.2f;
	public float 											stayAtConstant = 0.2f;
	public float 											stayOnAirTime = 0.1f;
	public float 											blinkTime = 2f;
	public float 											blinkTimeMissed = 4f;
	
	public int 												point;
	//public LayerMask										layerMask;
	public const float										stompMulti = 8f;
	public const float										dropMulti = 4f;
	
	private float											gravityMultiplier = 4f;
	private bool											isInvulnerable;
	
	private ControlKeys										_controls;
	public ControlKeys										Controls
	{
		set
		{
			if ( value != null && _controls != value )
			{
				_controls = value;
			}
		}
		get
		{
			return _controls;
		}
	}
	
	private tk2dAnimatedSprite 								_animSprite;
	public tk2dAnimatedSprite								AnimSprite
	{
		get
		{
			if ( _animSprite == null )
			{
				_animSprite = GetComponentInChildren< tk2dAnimatedSprite > ();
			}
			return _animSprite;
		}
	}
	
	private Transform 										_transform;
	public Transform										ThisTransform
	{
		get
		{
			if ( _transform == null )
			{
				_transform = transform;
			}
			return _transform;
		}
	}
	
	private CharacterController								_charControl;
	public CharacterController 								CharControl
	{
		get
		{
			if ( _charControl == null )
			{
				_charControl = GetComponentInChildren< CharacterController > ();
			}
			return _charControl;
		}
	}
	
	private Vector3											oriStandPos;
	private Vector3											lastGroundedPos;
	
	private static BetterList<PlayerController> 			players;
	
	private const float 									gravity = 100f;
	
	//----------------------------------------
	// public static methods
	//----------------------------------------
	public static PlayerController GetPlayer ( bool _player1 )
	{
		if ( players != null && players.size > 0 )
		{
			for ( int i=0; i<players.size; i++ )
			{
				if ( _player1 )
				{
					if ( players[i].ThisTransform.position.x < 0 )
					{
						return players[i];
					}
				}
				else
				{
					if ( players[i].ThisTransform.position.x > 0 )
					{
						return players[i];
					}
				}
			}
		}
		
		return null;
	}
	
	//----------------------------------------
	// public methods
	//----------------------------------------
	
	//----------------------------------------
	// private methods
	//----------------------------------------
	bool isJumping = false;
	private void Jump ()
	{
		if ( !CanJump ) return;
		
		Hashtable _jumpHash = new Hashtable ();
		_jumpHash.Add ( "name", "jumpTimer_" + gameObject.name );
		_jumpHash.Add ( "from", 0f );
		_jumpHash.Add ( "to", jumpHeight );
		_jumpHash.Add ( "time", jumpTime );
		_jumpHash.Add ( "easetype", iTween.EaseType.easeOutQuad );
		_jumpHash.Add ( "onupdate", "OnJumpUpdate" );
		iTween.ValueTo ( gameObject, _jumpHash );
		
		isJumping = true;
	}
	
	private void StopJump ()
	{
		iTween.StopByName ( "jumpTimer_" + gameObject.name );
		CanJump = false;
		isJumping = false;
		
		/*RaycastHit _hit;
		if ( Physics.Raycast ( ThisTransform.position, -Vector3.up, out _hit, layerMask ) )
		{
			if ( _hit.collider.gameObject != null &&
				_hit.collider.gameObject.CompareTag ( "Block" ) )
			{
				float _dist = Mathf.Abs ( _hit.point.y - ThisTransform.position.y );
				if ( _dist > 1f )
				{
					gravityMultiplier = Mathf.Max ( 1, _dist / GameController.tileSize * 3f );
				}
			}
		}*/
	}
	
	private void OnJumpUpdate ( float _value )
	{
		jumpValue = _value;
		if ( jumpValue >= jumpHeight)
		{
			StartCoroutine ( WaitThenStopJump ( stayOnAirTime ) );
		}
	}
	
	private IEnumerator WaitThenStopJump ( float _waitTime )
	{
		yield return new WaitForSeconds ( _waitTime );
		
		StopJump ();
	}
	
	bool waitMoveHor = false;
	private IEnumerator WaitThenSetMoveHorToNotConstant ()
	{
		waitMoveHor = true;
		
		int _dir = Mathf.RoundToInt ( AnimSprite.scale.x / AnimSprite.scale.x.Abs () );
		if ( _dir > 0 )
		{
			EffectController.Spawn ( "Stomp Effect", bodyParts.frontLeg.ThisTransform.position, _dir );
		}
		else
		{
			EffectController.Spawn ( "Stomp Effect", bodyParts.backLeg.ThisTransform.position, _dir );
		}
		AudioController.Play ( "Stomp" );
		
		AnimSprite.Play ( "Idle" );
		yield return new WaitForSeconds ( stayAtConstant );
		
		moveHorIsConstant = false;
		
		waitMoveHor = false;
	}
	
	public enum CollisionType { None, Head, Body, FrontLeg, BackLeg }
	private void CheckCollision ()
	{
		CollisionType _collision = CheckCollisionWithOpponent ();
		int _extraPoints = 0;
		if ( _collision != CollisionType.None )
		{
			switch ( _collision )
			{
			case CollisionType.Head:
				_extraPoints = GameController.HitPointHEAD;
				break;
				
			case CollisionType.Body:
				_extraPoints = GameController.HitPointBODY;
				break;
				
			case CollisionType.FrontLeg:
			case CollisionType.BackLeg:
				_extraPoints = GameController.HitPointLEG;
				break;
			}
			
			/*for ( int i=0; i<players.size; i++ )
			{
				if ( players[i] != this )
				{
					string _text = this.name + " STOMP\n" + players[i].name + "'s ";
					
					if ( _collision == CollisionType.BackLeg ||
						_collision == CollisionType.BackLeg )
					{
						_text += "LEG\n";
					}
					else
					{
						_text += _collision.ToString ().ToUpper () + "\n";
					}
					
					if ( _extraPoints > 0 )
					{
						_text += "+" + _extraPoints + " SCORES";
					}
					else
					{
						_text += _extraPoints + " SCORES";
					}
					
					UiGameController.ShowNews ( _text );
					break;
				}
			}*/
			
			point += _extraPoints;
			
			onPlayerScore ();
			return;
		}
	}
	
	private CollisionType CheckCollisionWithOpponent ()
	{
		// other than this player, there should also be another player
		if ( players != null && players.size > 1 )
		{
			foreach ( PlayerController _player in players.buffer )
			{
				if ( _player != null && _player != this 
					&& !_player.isInvulnerable && _player.bodyParts != null )
				{
					for ( int i=0; i<4; i++ )
					{
						SimpleBound _selectedBound = null;
						switch ( i )
						{
						case 0: // head
							_selectedBound = _player.bodyParts.head;
							break;
							
						case 1: // body
							_selectedBound = _player.bodyParts.body;
							break;
							
						case 2: // front leg
							_selectedBound = _player.bodyParts.frontLeg;
							break;
						
						case 3: // back leg
							_selectedBound = _player.bodyParts.backLeg;
							break;
						}
						
						if ( _selectedBound != null )
						{
							float _faceDir = AnimSprite.scale.x / AnimSprite.scale.x.Abs ();
							if ( _faceDir > 0 )
							{
								if ( CollidedWithSimpleBound ( bodyParts.frontLeg, _selectedBound ) )
								{
									CollisionType _type = ( CollisionType ) ( i + 1 );
									_player.GotHit ( _type );
									return _type;
								}
							}	
							else if ( _faceDir < 0 )
							{
								if ( CollidedWithSimpleBound ( bodyParts.backLeg, _selectedBound ) )
								{
									CollisionType _type = ( CollisionType ) ( i + 1 );
									_player.GotHit ( _type );
									return _type;
								}
							}
						}
					}
				}
			}
		}
		
		return CollisionType.None;
	}
	
	private bool CollidedWithSimpleBound ( SimpleBound _callerBound, SimpleBound _targetBound )
	{
		if ( _callerBound.boundType == SimpleBound.Type.Sphere &&
			_targetBound.boundType == SimpleBound.Type.Sphere )
		{
			float _dist = Vector3.Distance ( _callerBound.ThisTransform.position, _targetBound.ThisTransform.position );
			if ( _dist < ( _callerBound.boundRadius + _targetBound.boundRadius ) )
			{
				return true;
			}
		}
		else if ( _callerBound.boundType == SimpleBound.Type.Sphere &&
			_targetBound.boundType == SimpleBound.Type.Rectangle )
		{
			Vector3 _dir = ( _targetBound.ThisTransform.position - _callerBound.ThisTransform.position ).normalized;
			Vector3 _pointInDir = _callerBound.ThisTransform.position + _dir * _callerBound.boundRadius;
			Vector2 _pointInDirV2 = new Vector2 ( _pointInDir.x, _pointInDir.y );
			Rect _rectBound = new Rect 
				( 
					_targetBound.ThisTransform.position.x - _targetBound.boundRect.x / 2f,
					_targetBound.ThisTransform.position.y - _targetBound.boundRect.y / 2f,
					_targetBound.boundRect.x, _targetBound.boundRect.y
				);
			
			return ( _rectBound.Contains ( _pointInDirV2 ) );
		}
		else if ( _callerBound.boundType == SimpleBound.Type.Rectangle &&
			_targetBound.boundType == SimpleBound.Type.Sphere )
		{
			Vector3 _dir = ( _callerBound.ThisTransform.position - _targetBound.ThisTransform.position ).normalized;
			Vector3 _pointInDir = _targetBound.ThisTransform.position + _dir * _targetBound.boundRadius;
			Vector2 _pointInDirV2 = new Vector2 ( _pointInDir.x, _pointInDir.y );
			Rect _rectBound = new Rect 
				( 
					_callerBound.ThisTransform.position.x - _callerBound.boundRect.x / 2,
					_callerBound.ThisTransform.position.y - _callerBound.boundRect.y / 2,
					_callerBound.boundRect.x, _callerBound.boundRect.y
				);
			
			return ( _rectBound.Contains ( _pointInDirV2 ) );
		}
		
		return false;
	}
	
	private void GotHit ( CollisionType _collision )
	{
		isInvulnerable = true;
		
		switch ( _collision )
		{
		case CollisionType.Head:
		case CollisionType.Body:
			StartCoroutine ( InvulnerableBlink ( blinkTimeMissed ) );
			break;
			
		case CollisionType.FrontLeg:
		case CollisionType.BackLeg:
			StartCoroutine ( InvulnerableBlink ( blinkTime ) );
			break;
		}
		
		// hit effect
		SimpleBound _bound = null;
		switch ( _collision )
		{
		case CollisionType.Head:
			_bound = bodyParts.head;
			break;
			
		case CollisionType.Body:
			_bound = bodyParts.body;
			break;
			
		case CollisionType.FrontLeg:
			_bound = bodyParts.frontLeg;
			break;
			
		case CollisionType.BackLeg:
			_bound = bodyParts.backLeg;
			break;
		}
		
		if ( _bound != null )
		{
			int _dir = Mathf.RoundToInt ( AnimSprite.scale.x / AnimSprite.scale.x.Abs () );
			EffectController.Spawn ( "Hit Splash", _bound.ThisTransform.position, _dir );
			AudioController.Play ( "Hit" );
		}
	}
	
	private IEnumerator InvulnerableBlink ( float _blinkTime )
	{
		float _timer = _blinkTime;
		float _wait = 0.05f;
		while ( _timer > 0 )
		{
			AnimSprite.SetActive ( !AnimSprite.IsActive () );
			
			yield return new WaitForSeconds ( _wait );
			_timer -= _wait;
		}
		
		if ( !AnimSprite.IsActive () )
		{
			AnimSprite.SetActive ( true );
		}
		
		isInvulnerable = false;
	}
	
	bool isStomping = false;
	private IEnumerator JustStomp ()
	{
		isStomping = true;
		
		AnimSprite.PlayOnce ( "Stomp" );
		CheckCollision ();
		yield return new WaitForSeconds ( AnimSprite.CurrentClipLength () );
		
		isStomping = false;
	}
	
	//----------------------------------------
	// protected methods
	//----------------------------------------
	protected void Awake ()
	{
		if ( players == null )
		{
			players = new BetterList<PlayerController> ();
		}
		players.Add ( this );
		
		gameObject.name = name + players.size;
		
		gravityMultiplier = dropMulti;
		
		oriStandPos = ThisTransform.position;
		
		if ( ThisTransform.position.x < 0 )
		{
			MoveHor = 1f;
		}
		else 
		{
			MoveHor = -1f;
		}
	}
	
	protected void OnDestroy ()
	{
		if ( players != null )
		{
			players.Clear ();
			players = null;
		}
	}
	
	protected void OnDisable ()
	{
		if ( AnimSprite != null )
		{
			AnimSprite.SetActive ( false );
		}
	}
	
	protected void OnEnable ()
	{
		if ( AnimSprite != null )
		{
			AnimSprite.SetActive ( true );
		}
	}
	
	bool _canJump = false;
	bool CanJump
	{
		set
		{
			if ( _canJump != value )
			{//Debug.Log ( value );
				_canJump = value;
			}
		}
		get
		{
			return _canJump;
		}
	}
	bool isStayingOnAir = false;
	float jumpValue = 0f;
	float _moveHor = 0f;
	public float MoveHor
	{
		set
		{
			if ( _moveHor != value )
			{
				_moveHor = value;
				if ( _moveHor < 0 || _moveHor > 0 )
				{
					Vector3 _scale = AnimSprite.scale;
					_scale.x = Mathf.Abs ( _scale.x ) * _moveHor;
					AnimSprite.scale = _scale; 
				}
			}
		}
		get
		{
			return _moveHor;
		}
	}
	bool moveHorIsConstant = false;
	private void Update ()
	{
		#region Controls
		float _x = 0f;
		float _gravity = 0f;
		
		if ( !moveHorIsConstant )
		{
			MoveHor = 0;
		}
		
		if ( Controls != null && GameController.GameState == GameController.State.Game )
		{
			if ( !isStomping )
			{
				// move left and right
				if ( !moveHorIsConstant )
				{
					if ( Input.GetKey ( Controls.walkLeft ) )
					{
						MoveHor = -1f;
					}
					else if ( Input.GetKey ( Controls.walkRight ) )
					{
						MoveHor = 1f;
					}
				}
				
				if ( Input.GetKeyDown ( Controls.jump ) )
				{
					Jump ();
				}
				else if ( Input.GetKeyUp ( Controls.jump ) )
				{
					StopJump ();
				}
			}
			
			if ( Input.GetKey ( Controls.stomp ) )
			{
				if ( !moveHorIsConstant 
					&& ThisTransform.position.y == lastGroundedPos.y )
				{//Debug.LogError ( "stomp" );
					// just stomp
					if ( !isStomping )
					{
						StartCoroutine ( JustStomp () );
					}
				}
				
				if ( Input.GetKey ( Controls.walkLeft ) || Input.GetKey ( Controls.walkRight ) )
				{
					if ( !moveHorIsConstant && !isStomping
						&& ThisTransform.position.y != lastGroundedPos.y 
						)
					{
						gravityMultiplier = stompMulti;
						moveHorIsConstant = true;
						if ( Input.GetKey ( Controls.walkLeft ) ) 
						{
							MoveHor = -1f;
						}
						if ( Input.GetKey ( Controls.walkRight ) ) 
						{
							MoveHor = 1f;
						}
					}
				}
			}
		}
		#endregion
		
		if ( CharControl != null )
		{
			float _gravitySlow = 1f;
			if ( !isStomping )
			{
				if ( MoveHor < 0 || MoveHor > 0 )
				{
					if ( !CharControl.isGrounded )
					{
						if ( !moveHorIsConstant )
						{
							_x = speedOnAir * Time.deltaTime * MoveHor;
						}
						else
						{
							_x = speedOnAirConstant * Time.deltaTime * MoveHor;
							AnimSprite.PlayOnce ( "Stomp Drop" );
						}
					}
					else
					{
						_x = speed * Time.deltaTime * MoveHor;
						AnimSprite.PlayOnce ( "Walk" );
					}
					_gravitySlow = 0.75f;
				}
				else
				{
					if ( !moveHorIsConstant )
					{
						switch ( GameController.GameState )
						{
						case GameController.State.Game:
							AnimSprite.PlayOnce ( "Idle" );
							break;
						
						case GameController.State.End:
							if ( !AnimSprite.IsActive () )
							{
								AnimSprite.SetActive ( true );
							}
							
							if ( GameController.WinningPlayer.size < 2 )
							{
								if ( GameController.WinningPlayer[0] != this )
								{
									AnimSprite.PlayOnce ( "Lose" );
								}
								else
								{
									AnimSprite.PlayOnce ( "Idle" );
								}
							}
							else
							{
								// got two "winners"
								AnimSprite.PlayOnce ( "Lose" );
							}
							
							if ( ThisTransform.position.x != oriStandPos.x )
							{
								Vector3 _pos = ThisTransform.position;
								_pos.x = oriStandPos.x;
								ThisTransform.position = _pos;
								
								if ( oriStandPos.x < 0 )
								{
									MoveHor = 1f;
								}
								else
								{
									MoveHor = -1f;
								}
								
								isInvulnerable = false;
								StopAllCoroutines ();
							}
							break;
							
						default:
							AnimSprite.PlayOnce ( "Stand" );
							break;
						}
					}
				}
			}
			
			// jump
			if ( CanJump && !moveHorIsConstant )
			{
				Vector3 _pos = ThisTransform.position;
				_pos.y = lastGroundedPos.y + jumpValue;
				ThisTransform.position = _pos;
				
				if ( _pos.y != lastGroundedPos.y )
				{
					AnimSprite.PlayOnce ( "Jump" );
				}
			}
			
			// gravity
			if ( !CharControl.isGrounded )
			{
				if ( !isJumping )
				{
					RaycastHit _hit;
					if ( Physics.Raycast ( ThisTransform.position, -Vector3.up, out _hit ) )
					{
						if ( _hit.collider.gameObject != null &&
							_hit.collider.gameObject.CompareTag ( "Block" ) )
						{
							// check the distance of the hitPoint.y and ThisTransform.y to see if it exceed the smallest limit
							if ( Mathf.Abs ( _hit.point.y - ThisTransform.position.y ) > 1f )
							{
								CanJump = false;
							}
						}
					}
					
					_gravity = -gravity * Time.deltaTime * gravityMultiplier * _gravitySlow;
				}
				
				if ( moveHorIsConstant )
				{
					CheckCollision ();
				}
			}
			else
			{
				if ( !CanJump )
				{
					lastGroundedPos = ThisTransform.position;
					jumpValue = 0f;
					CanJump = true;
				}
				
				if ( gravityMultiplier == stompMulti )
				{
					iTween.ShakePosition ( GameController.MainCam.gameObject, new Vector3 ( 2f, 2f, 0f ), 1f );
					gravityMultiplier = dropMulti;
				}
				
				if ( moveHorIsConstant )
				{
					MoveHor = 0f;
					if ( !waitMoveHor )
					{
						StartCoroutine ( WaitThenSetMoveHorToNotConstant () );
					}
				}
			}
			
			CharControl.Move ( new Vector3 ( _x, _gravity, 0 ) );
		}
		
		// zeroed the z-axis
		Vector3 _posZ0 = ThisTransform.position;
		_posZ0.z = 0f;
		ThisTransform.position = _posZ0;
	}
	
}
