﻿using UnityEngine;
using System.Collections;

public class EffectController : MonoBehaviour 
{
	public string										uniqueName;
	public float 										waitTime;
	
	private int 										direction;
	private Transform									_transform;
	private Transform									ThisTransform
	{
		get
		{
			if ( _transform == null )
			{
				_transform = transform;
			}
			return _transform;
		}
	}
	
	private tk2dAnimatedSprite							_animSprite;
	public tk2dAnimatedSprite							AnimSprite
	{
		get
		{
			if ( _animSprite == null )
			{
				_animSprite = GetComponentInChildren< tk2dAnimatedSprite > ();
			}
			return _animSprite;
		}
	}
	
	private static BetterList<EffectController> 		effects;
	
	//----------------------------------------
	// PUBLIC STATIC methods
	//----------------------------------------
	public static void Spawn ( string _uniqueName, Vector3 _pos, int _direction )
	{
		if ( effects != null && effects.size > 0 )
		{
			foreach ( EffectController _effect in effects )
			{
				if ( !_effect.enabled 
					&& _effect.uniqueName.Equals ( _uniqueName )
					)
				{
					_effect.direction = _direction;
					_effect.enabled = true;
					_effect.ThisTransform.position = _pos;
					//_effect.ThisTransform.eulerAngles = new Vector3 ( 0, 0, Random.Range ( 0f, 360f ) );
					break;
				}
			}
		}
	}
	
	//----------------------------------------
	// PUBLIC methods
	//----------------------------------------
	
	//----------------------------------------
	// PRIVATE methods
	//----------------------------------------
	private IEnumerator PlayEffect ()
	{
		AnimSprite.Play ( uniqueName );
		yield return new WaitForSeconds ( AnimSprite.CurrentClipLength () );
		
		Color _color = AnimSprite.color;
		float _fadeTime = waitTime;
		while ( _color.a > 0 )
		{
			_color.a -= Time.deltaTime * ( 1f / _fadeTime );
			AnimSprite.color = _color;
			yield return null;
		}
		
		enabled = false;
	}
	
	//----------------------------------------
	// PROTECTED methods
	//----------------------------------------
	protected void Awake ()
	{
		if ( effects == null )
		{
			effects = new BetterList<EffectController> ();
		}
		effects.Add ( this );
		
		enabled = false;
	}
	
	protected void OnDisable ()
	{
		AnimSprite.color = Color.white;
		AnimSprite.SetActive ( false );
	}
	
	protected void OnEnable ()
	{
		AnimSprite.SetActive ( true );
		
		float _scaleX = AnimSprite.scale.x.Abs ();
		Vector3 _scale = AnimSprite.scale;
		_scale.x = _scaleX * direction;
		AnimSprite.scale = _scale;
		
		StartCoroutine ( PlayEffect () );
	}
	
}
