﻿using UnityEngine;
using System.Collections;

public class LoadObject
{
	public string name;
	public string path;
	public Vector3 position;
	public Vector3 rotation;
	public Vector3 scale;
	
	public LoadObject ()
	{
	}
	
	public LoadObject ( string _name, string _path, Vector3 _position )
	{
		name = _name;
		path = _path;
		position = _position;
		rotation = Vector3.zero;
		scale = Vector3.one;
	}
	
	public LoadObject ( string _name, string _path, Vector3 _position, Vector3 _rotation, Vector3 _scale )
	{
		name = _name;
		path = _path;
		position = _position;
		rotation = _rotation;
		scale = _scale;
	}
}

[ System.Serializable ]
public class ControlKeys
{
	public KeyCode walkLeft;
	public KeyCode walkRight;
	public KeyCode jump;
	public KeyCode stomp;
}

public class GameController: MonoBehaviour 
{
	public ControlKeys 						player1Controls;
	public ControlKeys						player2Controls;
	
	public int								_timeLimit;
	public int 								_scorePoint;
	public int 								_hitPointHEAD = 2;
	public int 								_hitPointBODY = 3;
	public int								_hitPointLEG = 4;
	
	public enum 							State
	{
		None,
		Menu,
		Setting,
		Countdown,
		Game,
		End
	}
	private State 							_gameState;
	
	private GameObject 						stageGo;
	private BetterList<LoadObject> 			loadObjects;
	private PlayerController				player1;
	private PlayerController				player2;
	private float 							countdownTimer;
	private int 							countdown;
	
	public enum LevelContent 
	{
		None,
		DefaultBlock,
		Jihab,
		Nazi
	}
	private BetterList<int[]>				levels;
	
	private static GameController 			mainGame;
	
	private const int 						column = 30;
	private const int 						row = 20;
	public const float 						tileSize = 20f;
	
	//----------------------------------------
	// public static methods
	//----------------------------------------
	int mode = -1;
	public static int Mode
	{
		set
		{
			if ( value >= 0 && 
				mainGame.mode != value )
			{
				mainGame.mode = value;
				
				mainGame.player1.MoveHor = 1;
				mainGame.player2.MoveHor = -1;
				
				switch ( mainGame.mode )
				{
				case 0: // NORMAL
					TimeLimit = 0;
					ScorePoint = 10; // 30
					HitPointHEAD = -2;
					HitPointBODY = -3;
					HitPointLEG = 4;
					break;	
				
				case 1: // SURVIVAL
					TimeLimit = 60;
					ScorePoint = -1; // infinite
					HitPointHEAD = -2;
					HitPointBODY = -3;
					HitPointLEG = 4;
					break;
					
				case 2: // MEXICAN STANDOFF
					TimeLimit = 10;
					ScorePoint = 4;
					HitPointHEAD = 0;
					HitPointBODY = 0;
					HitPointLEG = 4;
					
					mainGame.player1.MoveHor = -1;
					mainGame.player2.MoveHor = 1;
					break;
				}
				
				GameState = State.Countdown;
			}
		}
		get
		{
			return mainGame.mode;
		}
	}
	
	public static State GameState
	{
		set
		{
			if ( mainGame._gameState != value )
			{Debug.Log ( value );
				mainGame._gameState = value;
				
				switch ( mainGame._gameState )
				{
				case State.Menu:
					UiGameController.OpenPanel ( Panel.Type.MENU );
					Mode = -1;
					break;
					
				case State.Setting:
					UiGameController.OpenPanel ( Panel.Type.SETTING );
					break;
					
				case State.Countdown:
					UiGameController.OpenPanel ( Panel.Type.GAME );
					mainGame.countdown = -1;
					mainGame.countdownTimer = 0f;
					UiGameController.ChangeGameTimeLabel ( "" );
					UiGameController.DeactivateWinLabels ();
					mainGame.player1.point = 0;
					mainGame.player2.point = 0;
					UiGameController.SetPlayerScores ( mainGame.player1.name, mainGame.player2.name, mainGame.player1.point.ToString (), mainGame.player2.point.ToString () );
					
					if ( Mode <= 1 )
					{
						mainGame.player1.MoveHor = 1;
						mainGame.player2.MoveHor = -1;
					}
					else
					{
						mainGame.player1.MoveHor = -1;
						mainGame.player2.MoveHor = 1;
					}
					break;
					
				case State.Game:
					UiGameController.OpenPanel ( Panel.Type.GAME );
					if ( TimeLimit > 0 )
					{
						mainGame.countdown = TimeLimit;
						mainGame.countdownTimer = 0f;
						UiGameController.ChangeGameTimeLabel ( mainGame.countdown.ToString () );
					}
					break;
					
				case State.End:
					UiGameController.OpenPanel ( Panel.Type.GAME );
					UiGameController.DisplayWin ( mainGame.player1.point, mainGame.player2.point );
					mainGame.countdownTimer = 0f;
					break;
				}
			}
		}
		get
		{
			return mainGame._gameState;
		}
	}
	
	private Camera _mainCam;
	public static Camera MainCam
	{
		get
		{
			if ( mainGame._mainCam == null )
			{
				mainGame._mainCam = Camera.main;
			}
			return mainGame._mainCam;
		}
	}
	
	public static int TimeLimit
	{
		set
		{
			if ( mainGame._timeLimit != value )
			{
				mainGame._timeLimit = value;
			}
		}
		get
		{
			return mainGame._timeLimit;
		}
	}
	
	public static int ScorePoint
	{
		set
		{
			if ( mainGame._scorePoint != value )
			{
				mainGame._scorePoint = value;
			}
		}
		get
		{
			return mainGame._scorePoint;
		}
	}
	
	public static int HitPointHEAD
	{
		set
		{
			if ( mainGame._hitPointHEAD != value )
			{
				mainGame._hitPointHEAD = value;
			}
		}
		get
		{
			return mainGame._hitPointHEAD;
		}
	}
	
	public static int HitPointBODY
	{
		set
		{
			if ( mainGame._hitPointBODY != value )
			{
				mainGame._hitPointBODY = value;
			}
		}
		get
		{
			return mainGame._hitPointBODY;
		}
	}
	
	public static int HitPointLEG
	{
		set
		{
			if ( mainGame._hitPointLEG != value )
			{
				mainGame._hitPointLEG = value;
			}
		}
		get
		{
			return mainGame._hitPointLEG;
		}
	}
	
	public static BetterList<PlayerController> WinningPlayer
	{
		get
		{
			BetterList<PlayerController> _players = new BetterList<PlayerController> ();
			if ( mainGame.player1.point > mainGame.player2.point )
			{
				_players.Add ( mainGame.player1 );
			}
			else if ( mainGame.player2.point > mainGame.player1.point )
			{
				_players.Add ( mainGame.player2 );
			}
			else
			{
				_players.Add ( mainGame.player1 );
				_players.Add ( mainGame.player2 );
			}
			return _players;
		}
	}
	
	//----------------------------------------
	// public methods
	//----------------------------------------
	public void OnPlayerScore ()
	{
		if ( player1 == null || player2 == null ) return;
		
		UiGameController.SetPlayerScores ( player1.name, player2.name, player1.point.ToString (), player2.point.ToString () );
		
		if ( ScorePoint >= 0 ) // whoever hit first?
		{
			if ( player1.point >= ScorePoint || player2.point >= ScorePoint )
			{
				GameState = State.End;
			}
		}
	}
	
	//----------------------------------------
	// private methods
	//----------------------------------------
	private IEnumerator LoadAssets ()
	{
		string _basePath = "Prefabs/";
		
		int _loadCount = loadObjects.size;
		
		while ( _loadCount > 0 )
		{
			GameObject _go = ( GameObject ) Resources.Load ( _basePath + loadObjects[0].path + loadObjects[0].name );
			if ( _go != null )
			{
				GameObject _loaded = ( GameObject ) GameObject.Instantiate ( _go );
				_loaded.name = loadObjects[0].name;
				_loaded.transform.position = loadObjects[0].position;
				_loaded.transform.eulerAngles = loadObjects[0].rotation;
				_loaded.transform.localScale = loadObjects[0].scale;
				_loaded.transform.parent = stageGo.transform;
			}
			else
			{
				Debug.Log ( "'" + loadObjects[0].path + loadObjects[0].name + "' does not exist." );
			}
			
			loadObjects.RemoveAt ( 0 );
			
			_loadCount--;
			yield return null;
		}
		
		GameSetup ();
	}
	
	private void GameSetup ()
	{
		// setup players
		player1 = PlayerController.GetPlayer ( true );
		if ( player1 != null )
		{
			player1.Controls = player1Controls;
			player1.onPlayerScore += OnPlayerScore;
		}
		
		player2 = PlayerController.GetPlayer ( false );
		if ( player2 != null )
		{
			player2.Controls = player2Controls;
			player2.onPlayerScore += OnPlayerScore;
		}
		
		OnPlayerScore ();
		
		GameState = State.Menu;
	}
	
	//----------------------------------------
	// protected methods
	//----------------------------------------
	protected void Awake ()
	{
		mainGame = this;
		
		stageGo = new GameObject ( "Stage" );
		
		GameState = State.None;
		
		AudioController.Play ( "Theme" );
	}
	
	protected void OnDestroy ()
	{
		Destroy ( stageGo );
	}
	
	protected void Start () 
	{
		GameSetup ();
	}
	
	string[] countdownMessages = new string[4]
	{
		"THREE",
		"TWO",
		"ONE",
		"FIGHT~!!!"
	};
	protected void Update () 
	{
		if ( Input.GetKeyDown ( KeyCode.Escape ) )
		{
			Application.Quit ();
			return;
		}
		
		switch ( GameState )
		{
		case State.Menu:
			if ( Input.GetKeyDown ( KeyCode.Space ) )
			{
				GameState = State.Setting;
			}
			break;
			
		case State.Countdown:
			countdownTimer += Time.deltaTime;
			if ( countdownTimer >= 1f )
			{
				countdown++;
				if ( countdown >= countdownMessages.Length )
				{
					GameState = State.Game;
					UiGameController.ShowNews ( "" );
					return;
				}
				
				UiGameController.ShowNews 
					( 
						"-------------------------\n" +
						countdownMessages[ countdown ] + "\n" +
						"-------------------------\n"
						);
				
				countdownTimer = 0f;
			}
			break;
		
		case State.Game:
			if ( TimeLimit > 0 )
			{
				countdownTimer += Time.deltaTime;
				if ( countdownTimer >= 1f )
				{
					countdown--;
					if ( countdown < 0 )
					{
						GameState = State.End;
						return;
					}
					
					UiGameController.ChangeGameTimeLabel ( countdown.ToString () );
					
					countdownTimer = 0f;
				}
			}
			break;
			
		case State.End:
			if ( !UiGameController.PanelIsOpen ( Panel.Type.END ) )
			{
				countdownTimer += Time.deltaTime;
				if ( countdownTimer >= 3f )
				{
					UiGameController.OpenPanelSingle ( Panel.Type.END );
					if ( player1.point > player2.point )
					{
						UiGameController.ChangeGameOverLabel ( "Player 1 Won~!" );
					}
					else if ( player1.point < player2.point )
					{
						UiGameController.ChangeGameOverLabel ( "Player 2 Won~!" );
					}
					else
					{
						UiGameController.ChangeGameOverLabel ( "Too bad it's a draw..." );
					}
					countdownTimer = 0;
				}
			}
			break;
		}
	}
}
