﻿using UnityEngine;
using System.Collections;

[ System.Serializable ]
public class Panel
{
	public enum Type { MENU, SETTING, GAME, END }
	public Type type;
	public UIPanel panel;
}

public class UiGameController : MonoBehaviour 
{
	public UILabel									player1Name;
	public UILabel									player2Name;
	public UILabel									player1Scores;
	public UILabel									player2Scores;
	public UILabel									newsSource;
	public UILabel									player1WinCond;
	public UILabel									player2WinCond;
	public UILabel									gameTimeLabel;
	public UILabel									gameOverLabel;
	
	public UiButtonController						menuBtn;
	public UiButtonController						restartBtn;
	
	public UiButtonController						mode0;
	public UiButtonController						mode1;
	public UiButtonController						mode2;
	
	public Panel[] 									panels;
	
	private float 									newsSourceShowTimer;
	
	private static UiGameController					uiGame;
	
	//----------------------------------------
	// public static methods
	//----------------------------------------
	public static void OpenPanel ( Panel.Type _type )
	{
		if ( uiGame != null &&
			uiGame.panels != null && uiGame.panels.Length > 0 )
		{
			foreach ( Panel _panel in uiGame.panels )
			{
				if ( _panel.panel == null ) continue;
				
				if ( _panel.type == _type )
				{
					_panel.panel.gameObject.SetActive ( true );
				}
				else
				{
					_panel.panel.gameObject.SetActive ( false );
				}
			}
		}
	}
	
	public static void OpenPanelSingle ( Panel.Type _type )
	{
		if ( uiGame != null &&
			uiGame.panels != null && uiGame.panels.Length > 0 )
		{
			foreach ( Panel _panel in uiGame.panels )
			{
				if ( _panel.panel == null ) continue;
				
				if ( _panel.type == _type )
				{
					_panel.panel.gameObject.SetActive ( true );
					break;
				}
			}
		}
	}
	
	public static bool PanelIsOpen ( Panel.Type _type )
	{
		if ( uiGame != null &&
			uiGame.panels != null && uiGame.panels.Length > 0 )
		{
			foreach ( Panel _panel in uiGame.panels )
			{
				if ( _panel.panel == null ) continue;
				
				if ( _panel.type == _type )
				{
					return _panel.panel.gameObject.activeSelf;
				}
			}
		}
		
		return false;
	}
	
	public static void SetPlayerScores ( string _player1Name, string _player2Name, string _player1Scores, string _player2Scores )
	{
		if ( uiGame != null )
		{
			uiGame.player1Name.text = _player1Name;
			uiGame.player1Scores.text = _player1Scores;
			
			uiGame.player2Name.text = _player2Name;
			uiGame.player2Scores.text = _player2Scores;
		}
	}
	
	public static void ShowNews ( string _news )
	{
		if ( uiGame != null )
		{
			uiGame.newsSource.gameObject.SetActive ( true );
			uiGame.newsSource.text = _news;
			uiGame.newsSourceShowTimer = 3f;
		}
	}
	
	public static void DeactivateWinLabels ()
	{
		uiGame.player1WinCond.gameObject.SetActive ( false );
		uiGame.player2WinCond.gameObject.SetActive ( false );
	}
	
	public static void DisplayWin ( int _p1Scores, int _p2Scores )
	{
		uiGame.player1WinCond.gameObject.SetActive ( true );
		uiGame.player2WinCond.gameObject.SetActive ( true );
		
		if ( _p1Scores > _p2Scores )
		{
			uiGame.player1WinCond.text = "WIN!!!";
			uiGame.player2WinCond.text = "LOSE...";
		}
		else if ( _p1Scores < _p2Scores )
		{
			uiGame.player1WinCond.text = "LOSE...";
			uiGame.player2WinCond.text = "WIN!!!";
		}
		else
		{
			uiGame.player1WinCond.text = "IT'S A...";
			uiGame.player2WinCond.text = "DRAW...";
		}
	}
	
	public static void ChangeGameTimeLabel ( string _text )
	{
		uiGame.gameTimeLabel.text = _text;
	}
	
	public static void ChangeGameOverLabel ( string _text )
	{
		uiGame.gameOverLabel.text = _text;
	}
	
	//----------------------------------------
	// public methods
	//----------------------------------------
	
	//----------------------------------------
	// private methods
	//----------------------------------------
	private void OnButtonClick_MENU ()
	{
		if ( GameController.GameState == GameController.State.End )
		{
			GameController.GameState = GameController.State.Menu;
		}
	}
	
	private void OnButtonClick_RESTART ()
	{
		if ( GameController.GameState == GameController.State.End )
		{
			GameController.GameState = GameController.State.Countdown;
		}
	}
	
	private void OnButtonClick_Mode0 ()
	{
		GameController.Mode = 0;
	}
	
	private void OnButtonClick_Mode1 ()
	{
		GameController.Mode = 1;
	}
	
	private void OnButtonClick_Mode2 ()
	{
		GameController.Mode = 2;
	}
	
	//----------------------------------------
	// protected methods
	//----------------------------------------
	protected void Awake ()
	{
		uiGame = this;
			
		if ( panels != null && panels.Length > 0 )
		{
			foreach ( Panel _panel in panels )
			{
				if ( _panel.panel == null ) continue;
				
				_panel.panel.gameObject.SetActive ( false );
			}
		}
		
		DeactivateWinLabels ();
		
		menuBtn.onButtonClick += OnButtonClick_MENU;
		restartBtn.onButtonClick += OnButtonClick_RESTART;
		
		if ( mode0 != null ) mode0.onButtonClick += OnButtonClick_Mode0;
		if ( mode1 != null ) mode1.onButtonClick += OnButtonClick_Mode1;
		if ( mode2 != null ) mode2.onButtonClick += OnButtonClick_Mode2;
	}
	
	protected void OnDestroy ()
	{
		menuBtn.onButtonClick -= OnButtonClick_MENU;
		restartBtn.onButtonClick -= OnButtonClick_RESTART;
		
		if ( mode0 != null ) mode0.onButtonClick -= OnButtonClick_Mode0;
		if ( mode1 != null ) mode1.onButtonClick -= OnButtonClick_Mode1;
		if ( mode2 != null ) mode2.onButtonClick -= OnButtonClick_Mode2;
	}
	
	protected void Update ()
	{
		if ( PanelIsOpen ( Panel.Type.GAME ) )
		{
			if ( newsSource.gameObject.activeSelf )
			{
				newsSourceShowTimer -= Time.deltaTime;
				if ( newsSourceShowTimer <= 0 )
				{
					newsSource.text = "";
					newsSourceShowTimer = 0;
					newsSource.gameObject.SetActive ( false );
					return;
				}
			}
		}
	}
	
}
